package crm

import (
	"github.com/gin-gonic/gin"
	"hongze/fms_api/controller/resp"
	"hongze/fms_api/models/crm"
	crmService "hongze/fms_api/services/crm"
)

// CompanySellerController CRM销售
type CompanySellerController struct{}

// List
// @Title 销售列表
// @Description 销售列表
// @Success 200 {object} crm.SellerAdminWithGroupList
// @router /crm/company_seller/list [get]
func (rg *CompanySellerController) List(c *gin.Context) {
	// 此处调整为只取销售部门的人员, 而不是原先的根据角色取
	list, e := crmService.GetSellerDepartmentList()
	if e != nil {
		resp.FailData("获取失败", "获取销售列表失败, Err:"+e.Error(), c)
		return
	}
	resp.OkData("获取成功", list, c)
}

// GroupList
// @Title 销售组别列表
// @Description 销售组别列表
// @Success 200 {object} crm.SysGroup
// @router /crm/company_seller/group_list [get]
func (rg *CompanySellerController) GroupList(c *gin.Context) {
	groupOB := new(crm.SysGroup)
	cond := `department_id = ? AND parent_id = 0`
	pars := make([]interface{}, 0)
	pars = append(pars, crm.SellerDepartmentId)
	list, e := groupOB.List(cond, pars)
	if e != nil {
		resp.FailData("获取失败", "获取销售组别列表失败, Err:"+e.Error(), c)
		return
	}
	resp.OkData("获取成功", list, c)
}
